from telegram import Update, LabeledPrice, InlineKeyboardButton, InlineKeyboardMarkup
from telegram.ext import ApplicationBuilder, CommandHandler, CallbackQueryHandler, PreCheckoutQueryHandler, MessageHandler, filters, ContextTypes
import json

# Bot Token
BOT_TOKEN = 'YOUR_BOT_TOKEN_HERE'

# provider_token 留空，因为不涉及实物商品
PROVIDER_TOKEN = ''

# 存储商品信息和卡密
products = {}
card_keys = {}

# 存储管理员ID
ADMIN_ID = YOUR_ADMIN_ID_HERE

# Load data from file
def load_data():
    global products, card_keys
    try:
        with open('products.json', 'r') as f:
            products = json.load(f)
        with open('card_keys.json', 'r') as f:
            card_keys = json.load(f)
    except FileNotFoundError:
        products = {}
        card_keys = {}

# Save data to file
def save_data():
    with open('products.json', 'w') as f:
        json.dump(products, f)
    with open('card_keys.json', 'w') as f:
        json.dump(card_keys, f)

# Admin command to create a product
async def create_product(update: Update, context: ContextTypes.DEFAULT_TYPE):
    if update.effective_user.id != ADMIN_ID:
        return
    try:
        name, description, price = context.args
        price = int(price)
        products[name] = {'description': description, 'price': price}
        card_keys[name] = []
        save_data()
        await update.message.reply_text(f"商品 '{name}' 已创建！")
    except ValueError:
        await update.message.reply_text("请按照格式: /create_product 商品名 描述 价格")

# Admin command to add card keys to a product
async def add_card_keys(update: Update, context: ContextTypes.DEFAULT_TYPE):
    if update.effective_user.id != ADMIN_ID:
        return
    try:
        product_name = context.args[0]
        new_keys = context.args[1:]
        if product_name in card_keys:
            card_keys[product_name].extend(new_keys)
            save_data()
            await update.message.reply_text(f"卡密已添加到商品 '{product_name}'。")
        else:
            await update.message.reply_text("未找到该商品。")
    except IndexError:
        await update.message.reply_text("请按照格式: /add_card_keys 商品名 卡密1 卡密2 ...")

# Admin command to check the inventory of card keys
async def check_inventory(update: Update, context: ContextTypes.DEFAULT_TYPE):
    if update.effective_user.id != ADMIN_ID:
        return
    inventory = "\n".join([f"{k}: {len(v)} keys" for k, v in card_keys.items()])
    await update.message.reply_text(f"库存情况:\n{inventory}")

# User command to start and show product list
async def start(update: Update, context: ContextTypes.DEFAULT_TYPE):
    keyboard = [
        [InlineKeyboardButton(name, callback_data=name)] for name in products.keys()
    ]
    reply_markup = InlineKeyboardMarkup(keyboard)
    await update.message.reply_text("请选择您想购买的商品:", reply_markup=reply_markup)

# Handle product selection and initiate payment
async def product_selection(update: Update, context: ContextTypes.DEFAULT_TYPE):
    query = update.callback_query
    product_name = query.data
    await query.answer()

    product = products[product_name]
    title = f"购买 {product_name}"
    description = product['description']
    payload = f"purchase-{product_name}"
    currency = "XTR"  # 使用 Stars 进行支付
    prices = [LabeledPrice(f"{product_name} 卡密", product['price'])]

    # 发送支付信息
    await query.message.reply_invoice(
        title=title,
        description=description,
        payload=payload,
        provider_token=PROVIDER_TOKEN,
        currency=currency,
        prices=prices,
        start_parameter="card-purchase",
        is_flexible=False,
    )

# Handle pre-checkout (check payment info)
async def precheckout_callback(update: Update, context: ContextTypes.DEFAULT_TYPE):
    query = update.pre_checkout_query
    if query.invoice_payload.startswith("purchase-"):
        await query.answer(ok=True)
    else:
        await query.answer(ok=False, error_message="支付信息有误。")

# Handle successful payment
async def successful_payment_callback(update: Update, context: ContextTypes.DEFAULT_TYPE):
    product_name = update.message.successful_payment.invoice_payload.split("-")[1]
    if product_name in card_keys and card_keys[product_name]:
        card_key = card_keys[product_name].pop(0)  # 发送第一个卡密并移除
        save_data()
        await update.message.reply_text(f"感谢您的购买！您的卡密是: {card_key}")
    else:
        await update.message.reply_text(f"抱歉，'{product_name}' 的卡密已售罄。")

if __name__ == '__main__':
    load_data()
    application = ApplicationBuilder().token(BOT_TOKEN).build()

    # Admin commands
    application.add_handler(CommandHandler('create_product', create_product))
    application.add_handler(CommandHandler('add_card_keys', add_card_keys))
    application.add_handler(CommandHandler('check_inventory', check_inventory))

    # User commands
    application.add_handler(CommandHandler('start', start))
    application.add_handler(CallbackQueryHandler(product_selection))
    application.add_handler(PreCheckoutQueryHandler(precheckout_callback))
    application.add_handler(MessageHandler(filters.SUCCESSFUL_PAYMENT, successful_payment_callback))

    # Start bot
    application.run_polling()

